#!/usr/bin/env python 3

"""
Software Copyright License Agreement (BSD License)
The copyrights to the software code for SRST2 and accompanying code are licensed
under the following terms:
Copyright (c) 2013, Michael Inouye, Bernie Pope, Harriet Dashnow, Kathryn Holt
All rights reserved.
"""

from argparse import ArgumentParser
import xml.dom.minidom as xml
import urllib.request as url
from urllib.parse import urlparse
from olctools.accessoryFunctions.accessoryFunctions import make_path
import os

'''
Download MLST datasets from this site: http://pubmlst.org/data/ by
parsing an xml file (http://pubmlst.org/data/dbases.xml).
Data is downloaded for a species determined by the user:
- profiles (maps STs to allele numbers)
- numbered sequences for each locus in the scheme
In addition, the alleles are concatenated together for use with SRST2.
A log file is also generated in the working directory, detailing the
time, date and location of all files downloaded, as well as the <retrieved> 
tag which tells us when the XML entry was last updated. 
If the species name input by the user matches multiple <species> in the
xml file, the script simply reports the possible matches so the user can
try again.
'''


def parse_args():
    parser = ArgumentParser(description='Download MLST datasets by species'
                                        'from pubmlst.org.')

    parser.add_argument('--repository_url',
                        metavar='URL',
                        default='http://pubmlst.org/data/dbases.xml',
                        help='URL for MLST repository XML index')

    parser.add_argument('--genus',
                        metavar='NAME',
                        required=True,
                        help='The name of the species that you want to download (e.g. "Escherichia")')

    parser.add_argument('--force_scheme_name',
                        action="store_true",
                        default=False,
                        help='Flag to force downloading of specific scheme name (e.g. "Clostridium difficile")')

    parser.add_argument('--path',
                        metavar='PATH',
                        default=os.getcwd(),
                        help='Path in which to store the downloaded alleles and profiles')

    return parser.parse_args()


# test if a node is an Element and that it has a specific tag name
def testelementtag(node, name):
    return node.nodeType == node.ELEMENT_NODE and node.localName == name


# Get the text from an element node with a text node child
def gettext(element):
    result = ''
    for node in element.childNodes:
        if node.nodeType == node.TEXT_NODE:
            result += node.data
    return normalisetext(result)


# remove unwanted whitespace including linebreaks etc.
def normalisetext(string):
    return ' '.join(string.split())


# A collection of interesting information about a taxa
class SpeciesInfo(object):
    def __init__(self):
        self.name = None  # String name of species
        self.database_url = None  # URL as string
        self.retrieved = None  # date as string
        self.profiles_url = None  # URL as string
        self.profiles_count = None  # positive integer
        self.loci = []  # list of loci


class LocusInfo(object):
    def __init__(self):
        self.url = None
        self.name = None


# retrieve the interesting information for a given sample element
def getspeciesinfo(species_node, species, exact):
    this_name = gettext(species_node)
    store = False
    if exact:
        if this_name == species:
            store = True
    else:
        if this_name.startswith(species):
            store = True
    if store:
        info = SpeciesInfo()
        info.name = this_name
        for mlst_node in species_node.getElementsByTagName('mlst'):
            for database_node in mlst_node.getElementsByTagName('database'):
                for database_child_node in database_node.childNodes:
                    if testelementtag(database_child_node, 'url'):
                        info.database_url = gettext(database_child_node)
                    elif testelementtag(database_child_node, 'retrieved'):
                        info.retrieved = gettext(database_child_node)
                    elif testelementtag(database_child_node, 'profiles'):
                        for profile_count in database_child_node.getElementsByTagName('count'):
                            info.profiles_count = gettext(profile_count)
                        for profile_url in database_child_node.getElementsByTagName('url'):
                            info.profiles_url = gettext(profile_url)
                    elif testelementtag(database_child_node, 'loci'):
                        for locus_node in database_child_node.getElementsByTagName('locus'):
                            locus_info = LocusInfo()
                            locus_info.name = gettext(locus_node)
                            for locus_url in locus_node.getElementsByTagName('url'):
                                locus_info.url = gettext(locus_url)
                            info.loci.append(locus_info)
        return info
    else:
        return None


def main(args):
    # Create the path to store the schemes (if necessary)
    make_path(args.path)
    # Allow for Shigella to use the Escherichia MLST profile/alleles
    args.genus = args.genus if args.genus != 'Shigella' else 'Escherichia'
    # As there are multiple profiles for certain organisms, this dictionary has the schemes I use as values
    organismdictionary = {'Escherichia': 'Escherichia coli#1',
                          'Vibrio': 'Vibrio parahaemolyticus',
                          'Campylobacter': 'Campylobacter jejuni',
                          'Listeria': 'Listeria monocytogenes',
                          'Bacillus': 'Bacillus cereus',
                          'Staphylococcus': "Staphylococcus aureus",
                          'Salmonella': 'Salmonella enterica'}
    # Set the appropriate profile based on the dictionary key:value pairs
    try:
        args.genus = organismdictionary[args.species]
    except (KeyError, AttributeError):
        pass
    with url.urlopen(args.repository_url) as docfile:
        doc = xml.parse(docfile)
        root = doc.childNodes[0]
        found_species = []
        for species_node in root.getElementsByTagName('species'):
            info = getspeciesinfo(species_node, args.genus, args.force_scheme_name)
            if info is not None:
                found_species.append(info)
        if len(found_species) == 0:
            print("No species matched your query.")
            return
        if len(found_species) > 1:
            print("The following {} species match your query, please be more specific:".format(len(found_species)))
            for info in found_species:
                print(info.name)
                return
        # exit(2)

    # output information for the single matching species
    assert len(found_species) == 1
    species_info = found_species[0]
    species_name_underscores = species_info.name.replace(' ', '_')
    species_name_underscores = species_name_underscores.replace('/', '_')
    species_all_fasta_filename = species_name_underscores + '.fasta'
    species_all_fasta_file = open('{}/{}'.format(args.path, species_all_fasta_filename), 'w')
    log_filename = "mlst_data_download_{}_{}.log".format(species_name_underscores, species_info.retrieved)
    log_file = open('{}/{}'.format(args.path, log_filename), "w")
    log_file.write(species_info.retrieved + '\n')
    profile_path = urlparse(species_info.profiles_url).path
    profile_filename = profile_path.split('/')[-1]
    log_file.write("definitions: {}\n".format(profile_filename))
    log_file.write("{} profiles\n".format(species_info.profiles_count))
    log_file.write("sourced from: {}\n\n".format(species_info.profiles_url))
    #
    # with url.urlopen(species_info.profiles_url) as profile_doc:
    #     with open(os.path.join(args.path, profile_filename), 'w') as profile_file:
    localfile, headers = url.urlretrieve(species_info.profiles_url)
    with open(localfile, 'r') as profile_doc:
        with open(os.path.join(args.path, profile_filename), 'w') as profile_file:
            profile_file.write(profile_doc.read())
    for locus in species_info.loci:
        locus_path = urlparse(locus.url).path
        locus_filename = locus_path.split('/')[-1]
        log_file.write("locus {}\n".format(locus.name))
        log_file.write(locus_filename + '\n')
        log_file.write("Sourced from {}\n\n".format(locus.url))
        #
        local_locus_doc, headers = url.urlretrieve(locus.url)
        with open(local_locus_doc, 'r') as locus_doc:
            with open(os.path.join(args.path, locus_filename), 'w') as locus_file:
                # locus_doc = url.urlopen(locus.url)
                # locus_file = open('{}/{}'.format(args.path, locus_filename), 'w')
                locus_fasta_content = locus_doc.read()
                locus_file.write(locus_fasta_content)
                species_all_fasta_file.write(locus_fasta_content)
                # locus_file.close()
                # locus_doc.close()
    log_file.write("all loci: {}\n".format(species_all_fasta_filename))
    log_file.close()
    species_all_fasta_file.close()


if __name__ == '__main__':
    arguments = parse_args()
    main(arguments)



#!/usr/bin/env python3
from olctools.accessoryFunctions.accessoryFunctions import MetadataObject, GenObject, make_path, SetupLogging
from genemethods.typingclasses.typingclasses import GDCS, Resistance, Prophages, Serotype, Univec, Virulence
from genemethods.assemblypipeline.legacy_vtyper import Vtyper as LegacyVtyper
import olctools.accessoryFunctions.metadataprinter as metadataprinter
from genemethods.sixteenS.sixteens_full import SixteenS as SixteensFull
import genemethods.assemblypipeline.assembly_evaluation as evaluate
import genemethods.assemblypipeline.runMetadata as runMetadata
from genemethods.MLSTsippr.mlst import GeneSippr as MLSTSippr
from genemethods.assemblypipeline.basicAssembly import Basic
import genemethods.assemblypipeline.fastqmover as fastqmover
from genemethods.assemblypipeline.mobrecon import MobRecon
from genemethods.assemblypipeline.ec_typer import ECTyper
import genemethods.assemblypipeline.compress as compress
import genemethods.assemblypipeline.prodigal as prodigal
import genemethods.assemblypipeline.reporter as reporter
import genemethods.assemblypipeline.quality as quality
from genemethods.genesippr.genesippr import GeneSippr
import genemethods.assemblypipeline.sistr as sistr
import genemethods.assemblypipeline.skesa as skesa
from cowbat.metagenomefilter import automateCLARK
import genemethods.assemblypipeline.phix as phix
from genemethods.geneseekr.blast import BLAST
import genemethods.coreGenome.core as core
import genemethods.MASHsippr.mash as mash
from argparse import ArgumentParser
import multiprocessing
from time import time
import logging
import shutil
import os

__version__ = '0.5.0.6'
__author__ = 'adamkoziol'


class RunAssemble(object):

    def main(self):
        """
        Run the methods in the correct order
        """
        # Start the assembly
        self.helper()
        # Create the quality object
        self.create_quality_object()
        # Run the quality analyses
        self.quality()
        # Perform assembly
        self.assemble()
        # Perform genus-agnostic typing
        self.agnostictyping()
        # Perform typing
        self.typing()
        # Create a report
        reporter.Reporter(self)
        # Compress or remove all large, temporary files created by the pipeline
        compress.Compress(self)
        metadataprinter.MetadataPrinter(inputobject=self)

    def helper(self):
        """Helper function for file creation (if desired), manipulation, quality assessment,
        and trimming as well as the assembly"""
        # Simple assembly without requiring accessory files (SampleSheet.csv, etc).
        if self.basicassembly:
            self.runmetadata = Basic(inputobject=self)
        else:
            # Populate the runmetadata object by parsing the SampleSheet.csv, GenerateFASTQRunStatistics.xml, and
            # RunInfo.xml files
            self.runinfo = os.path.join(self.path, 'RunInfo.xml')
            self.runmetadata = runMetadata.Metadata(passed=self)
            # Extract the flowcell ID and the instrument name if the RunInfo.xml file was provided
            self.runmetadata.parseruninfo()
            # Extract PhiX mapping information from the run
            phi = phix.PhiX(inputobject=self)
            phi.main()
            # Populate the lack of bclcall and nohup call into the metadata sheet
            for sample in self.runmetadata.samples:
                sample.commands = GenObject()
                sample.commands.nohupcall = 'NA'
                sample.commands.bclcall = 'NA'
            # Move/link the FASTQ files to strain-specific working directories
            fastqmover.FastqMover(inputobject=self)
        # Print the metadata to file
        metadataprinter.MetadataPrinter(inputobject=self)

    def create_quality_object(self):
        """
        Create the quality object
        """
        self.qualityobject = quality.Quality(inputobject=self)

    def quality(self):
        """
        Creates quality objects and runs quality assessments and quality processes on the
        supplied sequences
        """
        # Validate that the FASTQ files are in the proper format, and that there are no issues e.g. different numbers
        # of forward and reverse reads, read length longer than quality score length, proper extension
        self.fastq_validate()
        # Run FastQC on the unprocessed fastq files
        self.fastqc_raw()
        # Perform quality trimming and FastQC on the trimmed files
        self.quality_trim()
        # Run FastQC on the trimmed files
        self.fastqc_trimmed()
        # Perform error correcting on the reads
        self.error_correct()
        # Detect contamination in the reads
        self.contamination_detection()
        # Run FastQC on the processed fastq files
        self.fastqc_trimmedcorrected()
        # Exit if only pre-processing of data is requested
        metadataprinter.MetadataPrinter(inputobject=self)
        if self.preprocess:
            logging.info('Pre-processing complete')
            quit()

    def fastq_validate(self):
        """
        Attempt to detect and fix issues with the FASTQ files
        """
        self.qualityobject.validate_fastq()
        metadataprinter.MetadataPrinter(inputobject=self)

    def fastqc_raw(self):
        """
        Run FastQC on the unprocessed FASTQ files
        """
        self.qualityobject.fastqcthreader(level='Raw')
        metadataprinter.MetadataPrinter(inputobject=self)

    def quality_trim(self):
        """
        Perform quality trimming and FastQC on the trimmed files
        """
        self.qualityobject.trimquality()
        metadataprinter.MetadataPrinter(inputobject=self)

    def fastqc_trimmed(self):
        """
        Run FastQC on the quality trimmed FASTQ files
        """
        self.qualityobject.fastqcthreader(level='Trimmed')
        metadataprinter.MetadataPrinter(inputobject=self)

    def error_correct(self):
        """
        Perform error correcting on the reads
        """
        self.qualityobject.error_correction()
        metadataprinter.MetadataPrinter(inputobject=self)

    def contamination_detection(self):
        """
        Calculate the levels of contamination in the reads
        """
        self.qualityobject.contamination_finder(report_path=self.reportpath)
        metadataprinter.MetadataPrinter(inputobject=self)

    def fastqc_trimmedcorrected(self):
        """
        Run FastQC on the processed fastq files
        """
        self.qualityobject.fastqcthreader(level='trimmedcorrected')
        metadataprinter.MetadataPrinter(inputobject=self)

    def assemble(self):
        """
        Assemble genomes and perform some basic quality analyses
        """
        # Assemble genomes
        self.assemble_genomes()
        # Calculate assembly metrics on raw assemblies
        self.evaluate_assemblies()
        # ORF detection
        self.prodigal()
        # Assembly quality determination
        self.genome_qaml()
        # CLARK analyses
        self.clark()

    def assemble_genomes(self):
        """
        Use skesa to assemble genomes
        """
        assembly = skesa.Skesa(inputobject=self)
        assembly.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def evaluate_assemblies(self):
        """
        Evaluate assemblies with Quast
        """
        qual = evaluate.AssemblyEvaluation(inputobject=self)
        qual.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def prodigal(self):
        """
        Use prodigal to detect open reading frames in the assemblies
        """
        prodigal.Prodigal(self)
        metadataprinter.MetadataPrinter(self)

    def genome_qaml(self):
        """
        Use GenomeQAML to determine the quality of the assemblies
        """
        g_qaml = quality.GenomeQAML(inputobject=self)
        g_qaml.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def clark(self):
        """
        Run CLARK metagenome analyses on the raw reads and assemblies if the system has adequate resources
        """
        # Run CLARK typing on the .fastq and .fasta files
        automateCLARK.PipelineInit(inputobject=self,
                                   extension='fasta',
                                   light=True
                                   )
        automateCLARK.PipelineInit(inputobject=self,
                                   extension='fastq',
                                   light=True)

    def agnostictyping(self):
        """
        Perform typing that does not require the genus of the organism to be known
        """
        # Run mash
        self.mash()
        # run rMLST on assemblies
        self.rmlst_assembled()
        # Run rMLST
        self.rmlst()
        # Run the 16S analyses
        self.sixteens()
        # Calculate the presence/absence of GDCS
        self.run_gdcs()
        # Find genes of interest
        self.genesippr()
        # Resistance finding - raw reads
        self.ressippr()
        # Resistance finding - assemblies
        self.resfinder()
        # Run MOB-suite
        self.mob_suite()
        # Prophage detection
        self.prophages()
        # Univec contamination search
        self.univec()
        # Virulence
        self.virulence()

    def mash(self):
        """
        Run mash to determine closest refseq genome
        """
        mash.Mash(inputobject=self,
                  analysistype='mash')
        metadataprinter.MetadataPrinter(inputobject=self)

    def rmlst_assembled(self):
        """
        Run rMLST analyses on assemblies
        """
        rmlst = BLAST(args=self,
                      analysistype='rmlst',
                      cutoff=100)
        rmlst.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)
        # Move the .rmlst attribute to .rmlst_assembled, so the raw reads can use .rmlst
        for sample in self.runmetadata.samples:
            sample.rmlst_assembled = GenObject()
            for key, value in sample.rmlst.datastore.items():
                setattr(sample.rmlst_assembled, key, value)
            sample.rmlst.datastore = {}
        # In order to keep the raw read rMLST analyses from having issues with the .rmlst attribute already existing,
        # copy the .rmlst attribute to .rmlst_assembled, and delete .rmlst from the object
        shutil.move(src=os.path.join(self.reportpath, 'rmlst.csv'),
                    dst=os.path.join(self.reportpath, 'rmlst_assembled.csv'))

    def rmlst(self):
        """
        Run rMLST analyses
        """
        rmlst = MLSTSippr(args=self,
                          pipelinecommit=self.commit,
                          startingtime=self.starttime,
                          scriptpath=self.homepath,
                          analysistype='rMLST',
                          pipeline=True,
                          cutoff=1.0)
        rmlst.runner()
        metadataprinter.MetadataPrinter(inputobject=self)

    def sixteens(self):
        """
        Run the 16S analyses
        """
        SixteensFull(args=self,
                     pipelinecommit=self.commit,
                     startingtime=self.starttime,
                     scriptpath=self.homepath,
                     analysistype='sixteens_full',
                     cutoff=0.95)
        metadataprinter.MetadataPrinter(inputobject=self)

    def run_gdcs(self):
        """
        Determine the presence of genomically-dispersed conserved sequences for Escherichia, Listeria, and Salmonella
        strains
        """
        # Run the GDCS analysis
        GDCS(inputobject=self)
        metadataprinter.MetadataPrinter(inputobject=self)

    def genesippr(self):
        """
        Find genes of interest
        """
        GeneSippr(args=self,
                  pipelinecommit=self.commit,
                  startingtime=self.starttime,
                  scriptpath=self.homepath,
                  analysistype='genesippr',
                  cutoff=0.95,
                  pipeline=False,
                  revbait=False)
        metadataprinter.MetadataPrinter(inputobject=self)

    def mob_suite(self):
        """

        """
        mob = MobRecon(metadata=self.runmetadata.samples,
                       analysistype='mobrecon',
                       databasepath=self.reffilepath,
                       threads=self.cpus,
                       logfile=self.logfile,
                       reportpath=self.reportpath)
        mob.mob_recon()
        metadataprinter.MetadataPrinter(inputobject=self)

    def ressippr(self):
        """
        Resistance finding - raw reads
        """
        res = Resistance(args=self,
                         pipelinecommit=self.commit,
                         startingtime=self.starttime,
                         scriptpath=self.homepath,
                         analysistype='resfinder',
                         cutoff=0.7,
                         pipeline=False,
                         revbait=True)
        res.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def resfinder(self):
        """
        Resistance finding - assemblies
        """
        resfinder = BLAST(args=self,
                          analysistype='resfinder_assembled')
        resfinder.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def prophages(self, cutoff=90):
        """
        Prophage detection
        :param cutoff: cutoff value to be used in the analyses
        """
        prophages = Prophages(args=self,
                              analysistype='prophages',
                              cutoff=cutoff,
                              unique=True)
        prophages.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def univec(self):
        """
        Univec contamination search
        """
        univec = Univec(args=self,
                        analysistype='univec',
                        cutoff=80,
                        unique=True)
        univec.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def virulence(self):
        """
        Virulence gene detection
        """
        vir = Virulence(args=self,
                        pipelinecommit=self.commit,
                        startingtime=self.starttime,
                        scriptpath=self.homepath,
                        analysistype='virulence',
                        cutoff=0.95,
                        pipeline=False,
                        revbait=True)
        if not os.path.isfile(os.path.join(self.reportpath, 'virulence.csv')):
            vir.reporter()
        metadataprinter.MetadataPrinter(inputobject=self)

    def typing(self):
        """
        Perform analyses that use genera-specific databases
        """
        # Run modules and print metadata to file
        # MLST
        self.mlst_assembled()
        # Assembly-based serotyping
        self.ec_typer()
        # Serotyping
        self.serosippr()
        # Assembly-based vtyper
        self.legacy_vtyper()
        # Core genome calculation
        self.coregenome()
        # Sistr
        self.sistr()

    def mlst_assembled(self):
        """
        Run rMLST analyses on assemblies
        """
        mlst = BLAST(args=self,
                     analysistype='mlst',
                     cutoff=100,
                     genus_specific=True)
        mlst.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def ec_typer(self):
        """
        Assembly-based serotyping
        """
        ec = ECTyper(metadata=self.runmetadata,
                     report_path=self.reportpath,
                     assembly_path=os.path.join(self.path, 'raw_assemblies'),
                     threads=self.cpus,
                     logfile=self.logfile)
        ec.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def serosippr(self):
        """
        Serotyping analyses
        """
        Serotype(args=self,
                 pipelinecommit=self.commit,
                 startingtime=self.starttime,
                 scriptpath=self.homepath,
                 analysistype='serosippr',
                 cutoff=0.90,
                 pipeline=True)
        metadataprinter.MetadataPrinter(inputobject=self)

    def legacy_vtyper(self):
        """
        Legacy vtyper - uses ePCR
        """
        legacy_vtyper = LegacyVtyper(inputobject=self,
                                     analysistype='legacy_vtyper',
                                     mismatches=2)
        legacy_vtyper.vtyper()
        metadataprinter.MetadataPrinter(inputobject=self)

    def coregenome(self):
        """
        Core genome calculation
        """
        coregen = core.CoreGenome(args=self,
                                  analysistype='coregenome',
                                  genus_specific=True)
        coregen.seekr()
        core.AnnotatedCore(inputobject=self)
        metadataprinter.MetadataPrinter(inputobject=self)

    def sistr(self):
        """
        Sistr
        """
        sistr_obj = sistr.Sistr(inputobject=self,
                                analysistype='sistr')
        sistr_obj.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def __init__(self, args):
        """
        Initialises the variables required for this class
        :param args: list of arguments passed to the script
        """
        SetupLogging()
        logging.info('Welcome to the CFIA de novo bacterial assembly pipeline {version}'
                     .format(version=__version__))
        # Define variables from the arguments - there may be a more streamlined way to do this
        self.args = args
        if args.sequencepath.startswith('~'):
            self.path = os.path.abspath(os.path.expanduser(os.path.join(args.sequencepath)))
        else:
            self.path = os.path.abspath(os.path.join(args.sequencepath))
        self.sequencepath = self.path
        if args.referencefilepath.startswith('~'):
            self.reffilepath = os.path.expanduser(os.path.abspath(os.path.join(args.referencefilepath)))
        else:
            self.reffilepath = os.path.abspath(os.path.join(args.referencefilepath))
        self.numreads = args.numreads
        self.preprocess = args.preprocess
        # Define the start time
        self.starttime = args.startingtime
        if args.customsamplesheet:
            if args.customsamplesheet.startswith('~'):
                self.customsamplesheet = os.path.expanduser(os.path.abspath(os.path.join(self.customsamplesheet)))
            else:
                self.customsamplesheet = os.path.abspath(os.path.join(args.customsamplesheet))
        else:
            self.customsamplesheet = args.customsamplesheet
        if self.customsamplesheet:
            assert os.path.isfile(self.customsamplesheet), 'Cannot find custom sample sheet as specified {css}'\
                .format(css=self.customsamplesheet)
        self.basicassembly = args.basicassembly
        if not self.customsamplesheet and not os.path.isfile(os.path.join(self.path, 'SampleSheet.csv')):
            self.basicassembly = True
            logging.warning('Could not find a sample sheet. Performing basic assembly (no run metadata captured)')
        # Use the argument for the number of threads to use, or default to the number of cpus in the system
        self.cpus = args.threads if args.threads else multiprocessing.cpu_count() - 1
        # Assertions to ensure that the provided variables are valid
        make_path(self.path)
        assert os.path.isdir(self.path), 'Supplied path location is not a valid directory {0!r:s}'.format(self.path)
        self.reportpath = os.path.join(self.path, 'reports')
        make_path(self.reportpath)
        assert os.path.isdir(self.reffilepath), 'Reference file path is not a valid directory {0!r:s}'\
            .format(self.reffilepath)
        self.commit = __version__
        self.homepath = args.homepath
        self.logfile = os.path.join(self.path, 'logfile')
        self.runinfo = str()
        self.pipeline = True
        self.qualityobject = MetadataObject()
        # Initialise the metadata object
        self.runmetadata = MetadataObject()


# If the script is called from the command line, then call the argument parser
if __name__ == '__main__':
    # Extract the path of the current script from the full path + file name
    homepath = os.path.split(os.path.abspath(__file__))[0]
    # Parser for arguments
    parser = ArgumentParser(description='Assemble genomes from Illumina fastq files')
    parser.add_argument('-v', '--version',
                        action='version', version='%(prog)s commit {}'.format(__version__))
    parser.add_argument('-s', '--sequencepath',
                        required=True,
                        help='Path to folder containing sequencing reads')
    parser.add_argument('-r', '--referencefilepath',
                        required=True,
                        help='Provide the location of the folder containing the pipeline accessory files (reference '
                             'genomes, MLST data, etc.')
    parser.add_argument('-n', '--numreads',
                        default=2,
                        type=int,
                        help='Specify the number of reads. Paired-reads:'
                        ' 2, unpaired-reads: 1. Default is paired-end')
    parser.add_argument('-t', '--threads',
                        help='Number of threads. Default is the number of cores in the system')
    parser.add_argument('-c', '--customsamplesheet',
                        help='Path of folder containing a custom sample sheet and name of sample sheet file '
                             'e.g. /home/name/folder/BackupSampleSheet.csv. Note that this sheet must still have the '
                             'same format of Illumina SampleSheet.csv files')
    parser.add_argument('-b', '--basicassembly',
                        action='store_true',
                        help='Performs a basic de novo assembly, and does not collect run metadata')
    parser.add_argument('-p', '--preprocess',
                        action='store_true',
                        help='Performs quality trimming and error correction only. Does not assemble the trimmed + '
                             'corrected reads')
    # Get the arguments into an object
    arguments = parser.parse_args()
    arguments.startingtime = time()
    arguments.homepath = homepath
    # Run the pipeline
    pipeline = RunAssemble(arguments)
    pipeline.main()
    logging.info('Assembly and characterisation complete')



#!/usr/bin/env python3
from olctools.accessoryFunctions.accessoryFunctions import MetadataObject, GenObject, make_path, relative_symlink, SetupLogging
from genemethods.assemblypipeline.legacy_vtyper import Vtyper as LegacyVtyper
import olctools.accessoryFunctions.metadataprinter as metadataprinter
from genemethods.assemblypipeline.typingclasses import Prophages, Univec
from genemethods.assemblypipeline.createobject import ObjectCreation
from genemethods.assemblypipeline.mobrecon import MobRecon
from cowbat.metagenomefilter import automateCLARK
import genemethods.assemblypipeline.prodigal as prodigal
import genemethods.assemblypipeline.reporter as reporter
import genemethods.assemblypipeline.quality as quality
import genemethods.assemblypipeline.sistr as sistr
from genemethods.geneseekr.blast import BLAST
import genemethods.coreGenome.core as core
import genemethods.MASHsippr.mash as mash
from argparse import ArgumentParser
import multiprocessing
from time import time
import logging
import os

__version__ = '0.0.01'
__author__ = 'adamkoziol'


class Typing(object):

    def main(self):
        """
        Run the methods in the correct order
        """
        # Create the metadata objects
        self.objects()
        # Determine assembly stats
        self.assembly_stats()
        # Perform genus-agnostic typing
        self.agnostictyping()
        # Perform typing
        self.typing()
        #
        self.typing_reports()
        # Compress or remove all large, temporary files created by the pipeline
        # compress.Compress(self)
        metadataprinter.MetadataPrinter(inputobject=self)

    def objects(self):
        """

        :return:
        """
        self.runmetadata = ObjectCreation(inputobject=self)
        make_path(os.path.join(self.path, 'BestAssemblies'))
        for sample in self.runmetadata.samples:
            # Link the assemblies to the BestAssemblies folder - necessary for GenomeQAML
            relative_symlink(sample.general.bestassemblyfile,
                             os.path.join(self.path, 'BestAssemblies'))
            # Create attributes required for downstream analyses
            sample.general.trimmedcorrectedfastqfiles = [sample.general.bestassemblyfile]
        # self.metadata = self.metadata.samples

    def assembly_stats(self):
        """
        Perform some basic quality analyses on the assemblies
        """
        # Calculate assembly metrics on polished assemblies
        self.quality_features(analysis='polished')
        # ORF detection
        self.prodigal()
        # Assembly quality determination
        self.genome_qaml()
        # CLARK analyses
        self.clark()
        metadataprinter.MetadataPrinter(inputobject=self)

    def quality_features(self, analysis):
        """
        Extract features from assemblies such as total genome size, longest contig, and N50
        """
        features = quality.QualityFeatures(inputobject=self,
                                           analysis=analysis)
        features.main()
        metadataprinter.MetadataPrinter(self)

    def prodigal(self):
        """
        Use prodigal to detect open reading frames in the assemblies
        """
        prodigal.Prodigal(self)
        metadataprinter.MetadataPrinter(self)

    def genome_qaml(self):
        """
        Use GenomeQAML to determine the quality of the assemblies
        """
        g_qaml = quality.GenomeQAML(inputobject=self)
        g_qaml.main()
        metadataprinter.MetadataPrinter(inputobject=self)

    def clark(self):
        """
        Run CLARK metagenome analyses on the raw reads and assemblies if the system has adequate resources
        """
        # Run CLARK typing on the .fastq and .fasta files
        automateCLARK.PipelineInit(inputobject=self,
                                   extension='fasta',
                                   light=True
                                   )

    def agnostictyping(self):
        """
        Perform typing that does not require the genus of the organism to be known
        """
        # Run mash
        self.mash()
        # Run rMLST
        self.rmlst()
        # Run the 16S analyses
        self.sixteens()
        # Calculate the presence/absence of GDCS
        self.run_gdcs()
        # Find genes of interest
        self.geneseekr()
        # Resistance finding - assemblies
        self.resfinder()
        # Run MOB-suite
        self.mob_suite()
        # Prophage detection
        self.prophages()
        # Univec contamination search
        self.univec()
        # Virulence
        self.virulence()

    def mash(self):
        """
        Run mash to determine closest refseq genome
        """
        logging.info('Running MASH analyses')
        mash.Mash(inputobject=self,
                  analysistype='mash')
        metadataprinter.MetadataPrinter(inputobject=self)

    def rmlst(self):
        """
        Run rMLST analyses
        """
        logging.info('rMLST')
        rmlst = BLAST(args=self,
                      analysistype='rMLST',
                      cutoff=100,
                      unique=True,
                      pipeline=True)
        rmlst.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def sixteens(self):
        """
        Run the 16S analyses
        """
        sixteen_s = BLAST(args=self,
                          analysistype='sixteens_full',
                          cutoff=95)
        sixteen_s.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def run_gdcs(self):
        """
        Determine the presence of genomically-dispersed conserved sequences for Escherichia, Listeria, and Salmonella
        strains
        """
        # Run the GDCS analysis
        gdcs = BLAST(args=self,
                     analysistype='GDCS',
                     cutoff=90,
                     genus_specific=True,
                     unique=False,)
        gdcs.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def geneseekr(self):
        """
        Find genes of interest
        """
        geneseekr = BLAST(args=self,
                          analysistype='genesippr',
                          cutoff=95)
        geneseekr.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def mob_suite(self):
        """

        """
        mob = MobRecon(metadata=self.runmetadata.samples,
                       analysistype='mobrecon',
                       databasepath=self.targetpath,
                       threads=self.cpus,
                       logfile=self.logfile,
                       reportpath=self.reportpath)
        mob.mob_recon()
        metadataprinter.MetadataPrinter(inputobject=self)

    def resfinder(self):
        """
        Resistance finding - assemblies
        """
        resfinder = BLAST(args=self,
                          analysistype='resfinder_assembled')
        resfinder.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def prophages(self):
        """
        Prophage detection
        """
        prophages = Prophages(args=self,
                              analysistype='prophages',
                              cutoff=90,
                              unique=True)
        prophages.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def univec(self):
        """
        Univec contamination search
        """
        univec = Univec(args=self,
                        analysistype='univec',
                        cutoff=80,
                        unique=True)
        univec.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def virulence(self):
        """
        Virulence gene detection
        """
        virulence = BLAST(args=self,
                          analysistype='virulence')
        virulence.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def typing(self):
        """
        Perform analyses that use genera-specific databases
        """
        # Run modules and print metadata to file
        # MLST
        self.mlst()
        # Serotyping
        self.serosippr()
        # Assembly-based vtyper
        self.legacy_vtyper()
        # Core genome calculation
        self.coregenome()
        # Sistr
        self.sistr()

    def mlst(self):
        """
         MLST analyses
        """
        logging.info('MLST')
        mlst = BLAST(args=self,
                     analysistype='MLST',
                     genus_specific=True,
                     cutoff=100,
                     unique=True)
        mlst.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def serosippr(self):
        """
        Serotyping analyses
        """
        #          pipeline=True)
        sero = BLAST(args=self,
                     analysistype='serosippr',
                     cutoff=90,
                     genus_specific=True,
                     unique=True)
        sero.seekr()
        metadataprinter.MetadataPrinter(inputobject=self)

    def legacy_vtyper(self):
        """
        Legacy vtyper - uses ePCR
        """
        legacy_vtyper = LegacyVtyper(inputobject=self,
                                     analysistype='legacy_vtyper',
                                     mismatches=2)
        legacy_vtyper.vtyper()
        metadataprinter.MetadataPrinter(inputobject=self)

    def coregenome(self):
        """
        Core genome calculation
        """
        coregen = core.CoreGenome(args=self,
                                  analysistype='coregenome',
                                  genus_specific=True)
        coregen.seekr()
        core.AnnotatedCore(inputobject=self)
        metadataprinter.MetadataPrinter(inputobject=self)

    def sistr(self):
        """
        Sistr
        """
        sistr.Sistr(inputobject=self,
                    analysistype='sistr')
        metadataprinter.MetadataPrinter(inputobject=self)

    def typing_reports(self):
        """
        Create empty attributes for analyses that were not performed, so that the metadata report can be created
        :return:
        """
        for sample in self.runmetadata.samples:
            sample.confindr = GenObject()
            sample.mapping = GenObject()
            sample.mapping.MeanInsertSize = 0
            sample.mapping.MeanCoveragedata = 0
            sample.mlst.results = sample.mlst.blastresults
            sample.genesippr.report_output = set()
            sample.genesippr.results = dict()
            try:
                for gene, percentid in sample.genesippr.blastresults.items():
                    if percentid > 95:
                        sample.genesippr.report_output.add(gene.split('_')[0])
            except AttributeError:
                sample.genesippr.report_output = list()
            sample.genesippr.report_output = sorted(list(sample.genesippr.report_output))
        # Create a report
        reporter.Reporter(inputobject=self,
                          legacy=False)

    def __init__(self, start, sequencepath, referencefilepath, scriptpath):
        """
        
        :param start: 
        :param sequencepath: 
        :param referencefilepath: 
        :param scriptpath:
        """
        SetupLogging(debug=True)
        logging.info('Welcome to the CFIA bacterial typing pipeline {}'
                     .format(__version__))
        # Define variables from the arguments - there may be a more streamlined way to do this
        self.sequencepath = os.path.join(sequencepath)
        self.path = self.sequencepath
        self.targetpath = os.path.join(referencefilepath)
        self.reffilepath = self.targetpath
        # Define the start time
        self.starttime = start
        self.start = self.starttime
        # Use the argument for the number of threads to use, or default to the number of cpus in the system
        self.cpus = multiprocessing.cpu_count() - 1
        # Assertions to ensure that the provided variables are valid
        assert os.path.isdir(self.sequencepath), 'Supplied path location is not a valid directory {0!r:s}'\
            .format(self.sequencepath)
        self.reportpath = os.path.join(self.sequencepath, 'reports')
        assert os.path.isdir(self.targetpath), 'Reference file path is not a valid directory {0!r:s}'\
            .format(self.targetpath)
        self.commit = __version__
        self.homepath = scriptpath
        self.analysistype = 'assembly_typing'
        self.genus_specific = False
        self.logfile = os.path.join(self.sequencepath, 'logfile')
        self.pipeline = True
        # Initialise the metadata object
        self.metadata = list()
        self.runmetadata = MetadataObject()


# If the script is called from the command line, then call the argument parser
if __name__ == '__main__':
    # Extract the path of the current script from the full path + file name
    homepath = os.path.split(os.path.abspath(__file__))[0]
    # Parser for arguments
    parser = ArgumentParser(description='Performing the typing component of the COWBAT pipeline on assemblies')
    parser.add_argument('-v', '--version',
                        action='version', version=__version__)
    parser.add_argument('-s', '--sequencepath',
                        required=True,
                        help='Path to folder containing sequencing reads')
    parser.add_argument('-r', '--referencefilepath',
                        required=True,
                        help='Provide the location of the folder containing the pipeline accessory files (reference '
                             'genomes, MLST data, etc.')
    arguments = parser.parse_args()
    # Run the pipeline
    pipeline = Typing(start=time(),
                      sequencepath=arguments.sequencepath,
                      referencefilepath=arguments.referencefilepath,
                      scriptpath=homepath)
    pipeline.main()
    logging.info('Characterisation complete')



#!/usr/bin/env python3
from olctools.accessoryFunctions.accessoryFunctions import GenObject, make_path, MetadataObject
import olctools.accessoryFunctions.metadataprinter as metadataprinter
from genemethods.assemblypipeline import createobject
from argparse import ArgumentParser
from threading import Thread
from subprocess import call
from csv import DictReader
from queue import Queue
import multiprocessing
from time import time
import subprocess
import logging
import os
__author__ = 'adamkoziol'


class FilterGenome(object):

    def objectprep(self):

        # Only find the data files if a datapath is provided
        if self.datapath:
            self.runmetadata = createobject.ObjectCreation(self)
        else:
            for sample in self.runmetadata.samples:
                sample.general.abundancefile = sample.general.abundance
                sample.general.assignmentfile = sample.general.classification
                sample.general.fastqfiles = [sample.general.combined]
        # Print the metadata to file
        metadataprinter.MetadataPrinter(self)
        # Load the results in the csv files into dictionaries
        self.taxids()

    def taxids(self):
        for sample in self.runmetadata.samples:
            # Initialise a list to store the taxIDs of interest
            sample.general.taxids = list()
            # Read the abundance file into a dictionary
            abundancedict = DictReader(open(sample.general.abundancefile))
            # Filter abundance to taxIDs with at least self.cutoff% of the total proportion
            for row in abundancedict:
                # The UNKNOWN category doesn't contain a 'Lineage' column, and therefore, subsequent columns are
                # shifted out of proper alignment, and do not contain the appropriate data
                try:
                    if float(row['Proportion_All(%)']) > self.cutoff:
                        sample.general.taxids.append(row['TaxID'], )
                except ValueError:
                    pass
            for taxid in sample.general.taxids:
                # Create the an attribute for each taxID
                setattr(sample, taxid, GenObject())
                sample[taxid].readlist = list()
        # Print the metadata to file
        metadataprinter.MetadataPrinter(self)
        # Load the assignment file to memory
        self.loadassignment()

    def loadassignment(self):
        """
        Load the taxonomic assignment for each read
        """
        logging.info('Finding taxonomic assignments')
        # Create and start threads
        for i in range(self.cpus):
            # Send the threads to the appropriate destination function
            threads = Thread(target=self.assignmentload, args=())
            # Set the daemon to true - something to do with thread management
            threads.setDaemon(True)
            # Start the threading
            threads.start()
        for sample in self.runmetadata.samples:
            self.loadqueue.put(sample)
        self.loadqueue.join()
        # Filter the .fastq files
        self.readlist()

    def assignmentload(self):
        while True:
            sample = self.loadqueue.get()
            # Initialise dictionaries to store results
            sample.general.fastqassignment = dict()
            sample.general.reads = dict()
            # Read the assignment file into a dictionary
            with open(sample.general.assignmentfile, 'r') as assignmentcsv:
                for row in assignmentcsv:
                    # Split on ','
                    data = row.split(',')
                    # Each row contains: Object_ID, Length, Assignment
                    # If the Assignment is one of the taxIDs of interest
                    if data[2].rstrip() in sample.general.taxids:
                        # Add the read name to the list of reads associated with that taxID
                        sample[data[2].rstrip()].readlist.append(data[0])
            self.loadqueue.task_done()

    def readlist(self):
        """
        Sort the reads, and create lists to be used in creating sorted .fastq files
        """
        logging.info('Sorting reads')
        # Create and start threads
        for i in range(self.cpus):
            # Send the threads to the appropriate destination function
            threads = Thread(target=self.listread, args=())
            # Set the daemon to true - something to do with thread management
            threads.setDaemon(True)
            # Start the threading
            threads.start()
        for sample in self.runmetadata.samples:
            self.listqueue.put(sample)
        self.listqueue.join()
        # Create
        self.fastqfilter()

    def listread(self):
        while True:
            sample = self.listqueue.get()
            # Set and create the path of the sorted fastq files
            sample.general.sortedfastqpath = os.path.join(sample.general.outputdirectory, 'sortedFastq')
            make_path(sample.general.sortedfastqpath)
            # Initialise dictionaries to hold data
            sample.general.fastqlist = dict()
            sample.general.filteredfastq = dict()
            # Iterate through the taxIDs
            for taxid in sample.general.taxids:
                # Set the name of the list to store all the reads associated with the taxID
                sample.general.fastqlist[taxid] = os.path.join(sample.general.sortedfastqpath,
                                                               '{sn}_{taxid}.txt'.format(sn=sample.name,
                                                                                         taxid=taxid))
                # Set the name of the .fastq file that will store the filtered reads
                sample.general.filteredfastq[taxid] = os.path.join(sample.general.sortedfastqpath,
                                                                   '{sn}_{taxid}.fastq.gz'.format(sn=sample.name,
                                                                                                  taxid=taxid))
                # Open the list, and write the list of all reads, one per line
                with open(sample.general.fastqlist[taxid], 'w') as binned:
                    binned.write('\n'.join(set(sample[taxid].readlist)))
            self.listqueue.task_done()

    def fastqfilter(self):
        """
        Filter the reads into separate files based on taxonomic assignment
        """
        logging.info('Creating filtered .fastqfiles')
        # Create and start threads
        for i in range(self.cpus):
            # Send the threads to the appropriate destination function
            threads = Thread(target=self.filterfastq, args=())
            # Set the daemon to true - something to do with thread management
            threads.setDaemon(True)
            # Start the threading
            threads.start()
        for sample in self.runmetadata.samples:
            self.filterqueue.put(sample)
        self.filterqueue.join()
        # Print the metadata to file
        metadataprinter.MetadataPrinter(self)

    def filterfastq(self):
        while True:
            sample = self.filterqueue.get()
            # Iterate through the taxIDs
            for taxid in sample.general.taxids:
                # Set the system call to seqtk to subsequence the fastq file
                sample.general.seqtkcall = 'seqtk subseq {fastq} {list} | gzip > {filtered}' \
                    .format(fastq=sample.general.fastqfiles[0],
                            list=sample.general.fastqlist[taxid],
                            filtered=sample.general.filteredfastq[taxid])
                # Run the system call only if the filtered file does not exist
                if not os.path.isfile(sample.general.filteredfastq[taxid]):
                    call(sample.general.seqtkcall, shell=True, stdout=self.devnull, stderr=self.devnull)
                # Delete the large list stored in the object
                delattr(sample[taxid], "readlist")
            self.filterqueue.task_done()

    def __init__(self, inputobject):
        # Define variables based on supplied arguments
        self.start = inputobject.start
        self.path = inputobject.path
        self.sequencepath = inputobject.sequencepath
        self.datapath = inputobject.datapath
        self.reportpath = inputobject.reportpath
        # Use the argument for the number of threads to use, or default to the number of cpus in the system
        self.cpus = inputobject.cpus
        # Set the cutoff to be a percent
        self.cutoff = inputobject.cutoff
        # Initialise a variable to hold the sample objects
        self.runmetadata = inputobject.runmetadata if inputobject.runmetadata else MetadataObject()
        # Initialise queues
        self.loadqueue = Queue()
        self.listqueue = Queue()
        self.filterqueue = Queue()
        self.devnull = open(os.devnull, 'wb')


# If the script is called from the command line, then call the argument parser
if __name__ == '__main__':
    class Parser(object):

        def __init__(self):

            # Get the current commit of the pipeline from git
            # Extract the path of the current script from the full path + file name
            homepath = os.path.split(os.path.abspath(__file__))[0]
            # Find the commit of the script by running a command to change to the directory containing the script and
            # run a git command to return the short version of the commit hash
            commit = subprocess.Popen('cd {} && git tag | tail -n 1'.format(homepath),
                                      shell=True, stdout=subprocess.PIPE).communicate()[0].rstrip()
            # Parser for arguments
            parser = ArgumentParser(description='Filter reads based on taxonomic assignment')
            parser.add_argument('-v', '--version',
                                action='version', version='%(prog)s commit {}'.format(commit))
            parser.add_argument('path',
                                help='Specify path')
            parser.add_argument('-t', '--threads',
                                help='Number of threads. Default is the number of cpus in the system')
            parser.add_argument('-s', '--sequencepath',
                                required=True,
                                help='Path of .fastq(.gz) files to process.')
            parser.add_argument('-d', '--datapath',
                                required=True,
                                help='Path of .csv files created by CLARK with read ID, length, and assignment.')
            parser.add_argument('-c', '--cutoff',
                                default=0.01,
                                help='Cutoff value for deciding which taxIDs to use when sorting .fastq files. '
                                     'Defaults to 1 percent. Please note that you must use a decimal format: enter 0.05'
                                     ' to get a 5 percent cutoff value')
            parser.add_argument('-x', '--taxids',
                                help='NOT IMPLEMENTED: CSV of desired taxIDs from each sample. ')
            # Get the arguments into an object
            args = parser.parse_args()
            self.start = time()
            # Define variables based on supplied arguments
            self.path = os.path.join(args.path)
            assert os.path.isdir(self.path), 'Supplied path is not a valid directory {path}'.format(path=self.path)
            self.sequencepath = os.path.join(args.sequencepath)
            assert os.path.isdir(self.sequencepath), 'Sequence location supplied is not a valid directory {seq_path}' \
                .format(seq_path=self.sequencepath)
            self.datapath = os.path.join(args.datapath)
            self.reportpath = os.path.join(self.path, 'reports')
            # Use the argument for the number of threads to use, or default to the number of cpus in the system
            self.cpus = args.threads if args.threads else multiprocessing.cpu_count()
            # Set the cutoff to be a percent
            self.cutoff = args.cutoff * 100
            # Run the pipeline
            self.runmetadata = MetadataObject()
            genome = FilterGenome(self)
            genome.objectprep()
            logging.info('Filtering complete')
    # Run the script
    Parser()


class PipelineInit(object):

    def __init__(self, inputobject):
        # Define variables based on supplied arguments
        self.start = inputobject.start
        self.path = inputobject.path
        self.sequencepath = inputobject.sequencepath
        self.datapath = inputobject.datapath
        self.reportpath = inputobject.reportpath
        # Use the argument for the number of threads to use, or default to the number of cpus in the system
        self.cpus = inputobject.cpus
        # Set the cutoff to be a percent
        self.cutoff = inputobject.cutoff
        # Initialise a variable to hold the sample objects
        self.runmetadata = inputobject.runmetadata
        # Initialise queues
        self.loadqueue = Queue()
        self.listqueue = Queue()
        self.filterqueue = Queue()
        self.devnull = open(os.devnull, 'wb')
        # Run the pipeline
        genome = FilterGenome(self)
        genome.objectprep()
        logging.info('Filtering complete')



#!/usr/bin/env python 3
from olctools.accessoryFunctions.accessoryFunctions import MetadataObject, GenObject, make_path
from genemethods.assemblypipeline import metadataReader
from cowbat.assembly_pipeline import RunAssemble
import multiprocessing
from time import time
import shutil
import os

testpath = os.path.abspath(os.path.dirname(__file__))
scriptpath = os.path.join(testpath, '..')
__author__ = 'adamkoziol'


def variables():
    v = MetadataObject()
    v.sequencepath = os.path.join(testpath, 'testdata')
    v.referencefilepath = os.path.join(v.sequencepath, 'databases')
    v.customsamplesheet = os.path.join(v.sequencepath, 'SampleSheet.csv')
    v.debug = True
    v.numreads = 2
    v.kmerrange = '21'
    v.preprocess = False
    v.basicassembly = True
    v.threads = multiprocessing.cpu_count()
    v.startingtime = time()
    v.commit = b''
    v.homepath = scriptpath
    return v


def method_init():
    global var
    var = variables()
    method_obj = RunAssemble(var)
    return method_obj


def read_metadata():
    metadata = metadataReader.MetadataReader()
    return metadata


method = method_init()


def test_sistr():
    metadata = MetadataObject()
    method.runmetadata.samples = list()
    fasta = os.path.join(var.sequencepath, 'NC_003198.fasta')
    metadata.name = os.path.split(fasta)[1].split('.')[0]
    # Initialise the general and run categories
    metadata.general = GenObject()
    metadata.run = GenObject()
    metadata.general.fastqfiles = list()
    # Set the destination folder
    outputdir = os.path.join(var.sequencepath, metadata.name)
    make_path(outputdir)
    # Add the output directory to the metadata
    metadata.general.outputdirectory = outputdir
    metadata.general.logout = os.path.join(outputdir, 'out')
    metadata.general.logerr = os.path.join(outputdir, 'err')
    metadata.run.outputdirectory = outputdir
    metadata.general.bestassemblyfile = True
    # Initialise an attribute to store commands
    metadata.commands = GenObject()
    # Assume that all samples are Salmonella
    metadata.general.referencegenus = 'Salmonella'
    # Set the .fasta file as the best assembly
    metadata.general.bestassemblyfile = fasta
    method.runmetadata.samples.append(metadata)
    method.sistr()
    for sample in method.runmetadata.samples:
        assert sample.sistr.cgmlst_genome_match == 'SAL_BA2732AA'
    variable_update()


def variable_update():
    global method
    method = method_init()


def test_basic_link():
    method.helper()
    assert os.path.islink(os.path.join(var.sequencepath, 'NC_002695', 'NC_002695_R1.fastq.gz'))


def test_metadata():
    method.helper()
    for sample in method.runmetadata.samples:
        assert sample.name == 'NC_002695'


def test_basic_read_length():
    method.helper()
    for sample in method.runmetadata.samples:
        assert sample.run.forwardlength == 301


def test_quality_object():
    method.create_quality_object()


def test_raw_fastqc_paired():
    method.fastqc_raw()
    for sample in method.runmetadata.samples:
        outfile = os.path.join(sample.general.outputdirectory, 'fastqc', 'Raw', 'NC_002695_fastqc.zip')
        size = os.stat(outfile)
        assert size.st_size > 0


def test_raw_fastqc_forward():
    for sample in method.runmetadata.samples:
        outfile = os.path.join(sample.general.outputdirectory, 'fastqc', 'Raw', 'NC_002695_R1_fastqc.zip')
        size = os.stat(outfile)
        assert size.st_size > 0


def test_quality_trim():
    method.quality_trim()
    outfile = os.path.join(var.sequencepath, 'NC_002695', 'NC_002695_R1_trimmed.fastq.gz')
    size = os.stat(outfile)
    assert size.st_size > 0


def test_trimmed_fastqc():
    method.fastqc_trimmed()
    for sample in method.runmetadata.samples:
        outfile = os.path.join(sample.general.outputdirectory, 'fastqc', 'Trimmed', 'NC_002695_R1_trimmed_fastqc.zip')
        size = os.stat(outfile)
        assert size.st_size > 0


def test_error_correction():
    method.error_correct()
    assert os.path.isfile(os.path.join(var.sequencepath, 'NC_002695', 'NC_002695_R1_trimmed_corrected.fastq.gz'))


def test_confindr():
    method.contamination_detection()
    for sample in method.runmetadata.samples:
        assert sample.confindr.num_contaminated_snvs == 0


def test_trimmed_corrected_fastqc():
    method.fastqc_trimmedcorrected()
    for sample in method.runmetadata.samples:
        outfile = os.path.join(sample.general.outputdirectory, 'fastqc', 'trimmedcorrected',
                               'NC_002695_R1_trimmed_corrected_fastqc.zip')
        size = os.stat(outfile)
        assert size.st_size > 0


def test_assemble_genomes():
    method.assemble_genomes()
    for sample in method.runmetadata.samples:
        outfile = os.path.join(sample.general.outputdirectory, 'assembly_output', 'NC_002695_unfiltered.fasta')
        size = os.stat(outfile)
        assert size.st_size > 0


def test_quality_features_raw():
    method.quality_features('raw')
    for sample in method.runmetadata.samples:
        assert sample.quality_features_raw.n50 >= 500


def test_qualimap():
    method.qualimap()
    for sample in method.runmetadata.samples:
        assert int(sample.mapping.Contigs) >= 500


def test_quality_features_polished():
    method.quality_features('polished')
    for sample in method.runmetadata.samples:
        assert sample.quality_features_polished.n50 >= 500


def test_prodigal():
    method.prodigal()
    for sample in method.runmetadata.samples:
        assert sample.prodigal.predictedgenesover1000bp >= 65


def test_mash():
    method.mash()
    for sample in method.runmetadata.samples:
        assert sample.mash.closestrefseq == 'Escherichia coli O157:H7 str. Sakai'


def test_rmlst():
    method.rmlst()
    for sample in method.runmetadata.samples:
        assert sample.rmlst.sequencetype == ['2124']


def test_sixteens():
    method.sixteens()
    for sample in method.runmetadata.samples:
        assert float(sample.sixteens_full.results['gi|219846739|ref|NR_026331.1|']) >= 99


def test_gdcs():
    method.run_gdcs()
    for sample in method.runmetadata.samples:
        assert os.path.isfile(sample.GDCS.baitedfastq)


def test_genesippr():
    method.genesippr()
    for sample in method.runmetadata.samples:
        assert sample.genesippr.results['VT1'] == '100.00'


def test_ressippr():
    method.ressippr()
    for sample in method.runmetadata.samples:
        assert float(sample.resfinder.avgdepth['sul1_1_AY224185']) >= 61.94


def test_resfinder():
    method.resfinder()
    for sample in method.runmetadata.samples:
        assert sample.resfinder_assembled.blastresults['sul1_1_AY224185'] >= 81


def test_prophages():
    method.prophages(cutoff=25)
    for sample in method.runmetadata.samples:
        assert sample.prophages.blastresults


def test_univec():
    method.univec()
    for sample in method.runmetadata.samples:
        assert sample.univec.blastresults


def test_virulence():
    method.virulence()
    for sample in method.runmetadata.samples:
        assert sample.virulence.snplocations['stx1:3:M19437:3'] == [504]


def test_mlst():
    method.mlst_assembled()
    for sample in method.runmetadata.samples:
        assert sample.mlst.sequencetype == ['11']


def test_ec_typer():
    method.ec_typer()
    for sample in method.runmetadata.samples:
        assert sample.ectyper.o_type == '-'
        assert sample.ectyper.h_type == 'H7'


def test_serosippr():
    method.serosippr()
    for sample in method.runmetadata.samples:
        assert sample.serosippr.o_set == ['O157']


def test_legacy_vtyper():
    method.legacy_vtyper()
    for sample in method.runmetadata.samples:
        assert 'vtx2f' in sample.legacy_vtyper.toxinprofile


def test_coregenome():
    method.coregenome()
    for sample in method.runmetadata.samples:
        assert sample.coregenome.coreresults == '1/1'


def test_clear_results():
    shutil.rmtree(os.path.join(var.sequencepath, 'NC_002695'))


def test_clear_sistr():
    shutil.rmtree(os.path.join(var.sequencepath, 'NC_003198'))


def test_clear_confindr():
    shutil.rmtree(os.path.join(var.sequencepath, 'confindr'))


def test_clear_reports():
    shutil.rmtree(os.path.join(var.sequencepath, 'reports'))


def test_clear_assemblies():
    shutil.rmtree(os.path.join(var.sequencepath, 'BestAssemblies'))


def test_clear_raw_assemblies():
    shutil.rmtree(os.path.join(var.sequencepath, 'raw_assemblies'))


def test_clear_kma():
    targetpath = os.path.join(var.referencefilepath, 'ConFindr', 'databases')
    os.remove(os.path.join(targetpath, 'Escherichia_db_kma.index.b'))
    os.remove(os.path.join(targetpath, 'Escherichia_db_kma.length.b'))
    os.remove(os.path.join(targetpath, 'Escherichia_db_kma.name'))
    os.remove(os.path.join(targetpath, 'Escherichia_db_kma.seq.b'))
    os.remove(os.path.join(targetpath, 'Escherichia_db_kma.comp.b'))


def test_clear_blastdb():
    targetpath = os.path.join(var.referencefilepath, 'MLST', 'Escherichia')
    os.remove(os.path.join(targetpath, 'combinedtargets.nhr'))
    os.remove(os.path.join(targetpath, 'combinedtargets.nin'))
    os.remove(os.path.join(targetpath, 'combinedtargets.nog'))
    os.remove(os.path.join(targetpath, 'combinedtargets.nsd'))
    os.remove(os.path.join(targetpath, 'combinedtargets.nsi'))
    os.remove(os.path.join(targetpath, 'combinedtargets.nsq'))


def test_clear_logs():
    os.remove(os.path.join(var.sequencepath, 'logfile_err.txt'))
    os.remove(os.path.join(var.sequencepath, 'logfile_out.txt'))



#!/usr/bin/env python
from setuptools import setup, find_packages
import os
__author__ = 'adamkoziol'
setup(
    name="COWBAT",
    version="0.5.0.6",
    include_package_data=True,
    packages=find_packages(),
    scripts=[os.path.join('cowbat', 'assembly_pipeline.py'),
             os.path.join('cowbat', 'assembly_typing.py'),
             os.path.join('cowbat', 'validation', 'validate_cowbat.py')
             ],
    license='MIT',
    author='Adam Koziol',
    author_email='adam.koziol@canada.ca',
    description='CFIA OLC Workflow for Bacterial Assembly and Typing',
    url='https://github.com/OLC-Bioinformatics/COWBAT',
    long_description=open('README.md').read(),
    install_requires=['interop']
)
